@@ -10,7 +10,7 @@ my $stream = Data::BitStream->new();
 die unless defined $stream;
 Data::BitStream::Code::Escape->meta->apply($stream);
 
-my $p = 2;
+my $p = 0;
 while (<>) {
   chomp;
   # Allows setting the parameter via:  p=....
@@ -22,23 +22,15 @@ while (<>) {
   my $v = $_;
 
   $stream->erase_for_write;
-  #$stream->put_baer($p, $v);
+  $stream->put_baer($p, $v);
   #$stream->put_escape($p, $v);
-  #$stream->put_blocktaboo($p, $v);
-  #$stream->put_fibgen($p, $v);
-  $stream->put_comma($p, $v);
-  #$stream->put_goldbach_g2($v);
 
   my $s = $stream->to_string;
   print "        $s\n";
 
   $stream->rewind_for_read;
-  #my $d = $stream->get_baer($p);
+  my $d = $stream->get_baer($p);
   #my $d = $stream->get_escape($p);
-  #my $d = $stream->get_blocktaboo($p);
-  #my $d = $stream->get_fibgen($p);
-  my $d = $stream->get_comma($p);
-  #my $d = $stream->get_goldbach_g2();
   if ($d != $v) {
     print "DECODED:  $d instead of $v\n";
   }
