@@ -26,10 +26,11 @@ requires 'read', 'write', 'put_unary', 'get_unary';
 sub put_baer {
   my $self = shift;
   my $k = shift;
-  die "invalid parameters" if $k > 32 || $k < -32;
+  $self->error_code('param', 'k must be between -32 and 32') if $k > 32 || $k < -32;
   my $mk = ($k < 0) ? int(-$k) : 0;
 
   foreach my $v (@_) {
+    $self->error_code('zeroval') unless defined $v and $v >= 0;
     if ($v < $mk) {
       $self->put_unary1($v);
       next;
@@ -63,7 +64,7 @@ sub put_baer {
 sub get_baer {
   my $self = shift;
   my $k = shift;
-  die "invalid parameters" if $k > 32 || $k < -32;
+  $self->error_code('param', 'k must be between -32 and 32') if $k > 32 || $k < -32;
   my $mk = ($k < 0) ? int(-$k) : 0;
 
   my $count = shift;
@@ -72,7 +73,10 @@ sub get_baer {
   elsif ($count == 0)     { return;      }
 
   my @vals;
+  my $maxbits = $self->maxbits;
+  $self->code_pos_start('Baer');
   while ($count-- > 0) {
+    $self->code_pos_set;
     my $C = $self->get_unary1;
     last unless defined $C;
     if ($C < $mk) {
@@ -80,6 +84,7 @@ sub get_baer {
       next;
     }
     $C -= $mk;
+    $self->error_code('overflow') if $C > $maxbits;
     my $val = ($self->read(1) == 0)  ?  1  :  2 + $self->read(1);
 
     # Code following the logic in the paper:
@@ -97,6 +102,7 @@ sub get_baer {
 
     push @vals, $val;
   }
+  $self->code_pos_end;
   wantarray ? @vals : $vals[-1];
 }
 no Mouse::Role;
