@@ -43,7 +43,7 @@ sub put_levenstein {
   my $self = shift;
 
   foreach my $v (@_) {
-    die "Value must be >= 0" unless $v >= 0;
+    $self->error_code('zeroval') unless defined $v and $v >= 0;
     if ($v == 0) { $self->write(1, 0); next; }
 
     # Simpler code:
@@ -91,25 +91,35 @@ if (0) {
 
 sub get_levenstein {
   my $self = shift;
+  $self->error_stream_mode('read') if $self->writing;
   my $count = shift;
   if    (!defined $count) { $count = 1;  }
   elsif ($count  < 0)     { $count = ~0; }   # Get everything
   elsif ($count == 0)     { return;      }
 
   my @vals;
+  my $maxbits = $self->maxbits;
+  $self->code_pos_start('Levenstein');
   while ($count-- > 0) {
+    $self->code_pos_set;
+
     my $C = $self->get_unary1;
     last unless defined $C;
+
     my $val = 0;
     if ($C > 0) {
       my $N = 1;
       for (1 .. $C-1) {
-        $N = (1 << $N) | $self->read($N);
+        $self->error_code('overflow') if $N > $maxbits;
+        my $next = $self->read($N);
+        $self->error_off_stream unless defined $next;
+        $N = (1 << $N) | $next;
       }
       $val = $N;
     }
     push @vals, $val;
   }
+  $self->code_pos_end;
   wantarray ? @vals : $vals[-1];
 }
 no Mouse::Role;
@@ -119,6 +129,8 @@ no Mouse::Role;
 
 =pod
 
+=encoding utf8
+
 =head1 NAME
 
 Data::BitStream::Code::Levenstein - A Role implementing Levenstein codes
@@ -185,11 +197,11 @@ These methods are required for the role.
 
 =head1 AUTHORS
 
-Dana Jacobsen <dana@acm.org>
+Dana Jacobsen E<lt>dana@acm.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2011 by Dana Jacobsen <dana@acm.org>
+Copyright 2011 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
