@@ -3,7 +3,7 @@ use strict;
 use warnings;
 BEGIN {
   $Data::BitStream::MinimalVec::AUTHORITY = 'cpan:DANAJ';
-  $Data::BitStream::MinimalVec::VERSION   = '0.01';
+  $Data::BitStream::MinimalVec::VERSION   = '0.02';
 }
 
 use Mouse;
@@ -22,6 +22,9 @@ with 'Data::BitStream::Base',
      'Data::BitStream::Code::Baer',
      'Data::BitStream::Code::BoldiVigna',
      'Data::BitStream::Code::ARice',
+     'Data::BitStream::Code::Additive',
+     'Data::BitStream::Code::Comma',
+     'Data::BitStream::Code::Taboo',
      'Data::BitStream::Code::StartStop';
 
 has '_vec' => (is => 'rw', default => '');
@@ -31,14 +34,16 @@ after 'erase' => sub { shift->_vec(''); 1; };
 
 sub read {
   my $self = shift;
-  die "read while writing" if $self->writing;
+  $self->error_stream_mode('read') if $self->writing;
   my $bits = shift;
-  die "invalid bits: $bits" unless defined $bits && $bits > 0 && $bits <= $self->maxbits;
+  $self->error_code('param', 'bits must be in range 1-' . $self->maxbits)
+         unless defined $bits && $bits > 0 && $bits <= $self->maxbits;
   my $peek = (defined $_[0]) && ($_[0] eq 'readahead');
 
   my $pos = $self->pos;
   my $len = $self->len;
   return if $pos >= $len;
+  $self->error_off_stream if !$peek && ($pos+$bits) > $len;
 
   my $val = 0;
   my $rvec = $self->_vecref;
@@ -50,11 +55,11 @@ sub read {
 }
 sub write {
   my $self = shift;
-  die "write while reading" unless $self->writing;
+  $self->error_stream_mode('write') unless $self->writing;
   my $bits = shift;
-  die "invalid bits: $bits" unless defined $bits && $bits > 0;
+  $self->error_code('param', 'bits must be > 0') unless defined $bits && $bits > 0;
   my $val  = shift;
-  die "undefined value" unless defined $val;
+  $self->error_code('zeroval') unless defined $val and $val >= 0;
 
   my $len  = $self->len;
   my $rvec = $self->_vecref;
@@ -64,7 +69,7 @@ sub write {
   } elsif ($val == 1) {
     vec($$rvec, $len + $bits - 1, 1) = 1;
   } else {
-    die "invalid bits: $bits" if $bits > $self->maxbits;
+    $self->error_code('param', 'bits must be <= ' . $self->maxbits) if $bits > $self->maxbits;
 
     my $wpos = $len + $bits-1;
     foreach my $bit (0 .. $bits-1) {
@@ -170,6 +175,18 @@ The following roles are included.
 
 =item L<Data::BitStream::Code::StartStop>
 
+=item L<Data::BitStream::Code::Baer>
+
+=item L<Data::BitStream::Code::BoldiVigna>
+
+=item L<Data::BitStream::Code::ARice>
+
+=item L<Data::BitStream::Code::Additive>
+
+=item L<Data::BitStream::Code::Comma>
+
+=item L<Data::BitStream::Code::Taboo>
+
 =back
 
 =head1 SEE ALSO
@@ -186,11 +203,11 @@ The following roles are included.
 
 =head1 AUTHORS
 
-Dana Jacobsen <dana@acm.org>
+Dana Jacobsen E<lt>dana@acm.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2011 by Dana Jacobsen <dana@acm.org>
+Copyright 2011-2012 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
